@@ -263,7 +263,10 @@ class DFMessage(object):
                     v = v.tostring()
             else:
                 if isinstance(v,str):
-                    v = bytes(v,'ascii')
+                    try:
+                        v = bytes(v,'ascii')
+                    except UnicodeEncodeError:
+                        v = v.encode()
                 elif isinstance(v, array.array):
                     v = v.tobytes()
             if mul is not None:
@@ -483,7 +486,11 @@ class DFReader(object):
         self.verbose = False
         self.params = {}
         self._flightmodes = None
-        self.messages = {}
+        self.messages = {
+            'MAV': self,
+            '__MAV__': self,  # avoids conflicts with messages actually called "MAV"
+        }
+        self.percent = 0
 
     def _rewind(self):
         '''reset state on rewind'''
@@ -492,7 +499,10 @@ class DFReader(object):
         # need their messages to disappear to.  If they want their own
         # copy they can copy.copy it!
         self.messages.clear()
-        self.messages['MAV'] = self
+        self.messages = {
+            'MAV': self,
+            '__MAV__': self,  # avoids conflicts with messages actually called "MAV"
+        }
         if self._flightmodes is not None and len(self._flightmodes) > 0:
             self.flightmode = self._flightmodes[0][0]
         else:
@@ -813,6 +823,7 @@ class DFReader_binary(DFReader):
                 if not idata in type_instances[mtype]:
                     # its a new one, need to parse it so we have the complete set of instances
                     type_instances[mtype].add(idata)
+                    self.offset = ofs
                     self._parse_next()
 
             self.counts[mtype] += 1
@@ -890,7 +901,7 @@ class DFReader_binary(DFReader):
         if self.type_nums is None:
             # always add some key msg types so we can track flightmode, params etc
             type = type.copy()
-            type.update(set(['MODE','MSG','PARM','STAT']))
+            type.update(set(['MODE','MSG','PARM','STAT','ORGN']))
             self.indexes = []
             self.type_nums = []
             for t in type:
@@ -1128,7 +1139,7 @@ class DFReader_text(DFReader):
         if self.type_list is None:
             # always add some key msg types so we can track flightmode, params etc
             self.type_list = type.copy()
-            self.type_list.update(set(['MODE','MSG','PARM','STAT']))
+            self.type_list.update(set(['MODE','MSG','PARM','STAT','ORGN']))
             self.type_list = list(self.type_list)
             self.indexes = []
             self.type_nums = []
