@@ -14,6 +14,8 @@ import copy
 import re
 from pymavlink import mavexpression
 
+is_py3 = sys.version_info >= (3,0)
+
 # adding these extra imports allows pymavlink to be used directly with pyinstaller
 # without having complex spec files. To allow for installs that don't have ardupilotmega
 # at all we avoid throwing an exception if it isn't installed
@@ -66,14 +68,16 @@ def evaluate_condition(condition, vars):
     return v
 
 def u_ord(c):
-	return ord(c) if sys.version_info.major < 3 else c
+    if is_py3:
+        return c
+    return ord(c)
 
 class location(object):
     '''represent a GPS coordinate'''
     def __init__(self, lat, lng, alt=0, heading=0):
-        self.lat = lat
-        self.lng = lng
-        self.alt = alt
+        self.lat = lat  # in degrees
+        self.lng = lng  # in degrees
+        self.alt = alt  # in metres
         self.heading = heading
 
     def __str__(self):
@@ -457,7 +461,10 @@ class mavfile(object):
 
             if numnew != 0:
                 if self.logfile_raw:
-                    self.logfile_raw.write(str(s))
+                    if is_py3:
+                        self.logfile_raw.write(s)
+                    else:
+                        self.logfile_raw.write(str(s))
                 if self.first_byte:
                     self.auto_mavlink_version(s)
 
@@ -467,7 +474,10 @@ class mavfile(object):
             if msg:
                 if self.logfile and  msg.get_type() != 'BAD_DATA' :
                     usec = int(time.time() * 1.0e6) & ~3
-                    self.logfile.write(str(struct.pack('>Q', usec) + msg.get_msgbuf()))
+                    if is_py3:
+                        self.logfile.write(struct.pack('>Q', usec) + msg.get_msgbuf())
+                    else:
+                        self.logfile.write(str(struct.pack('>Q', usec) + msg.get_msgbuf()))
                 self.post_message(msg)
                 return msg
             else:
@@ -518,11 +528,11 @@ class mavfile(object):
         '''return True if using MAVLink 2.0 or later'''
         return float(self.WIRE_PROTOCOL_VERSION) >= 2
 
-    def setup_logfile(self, logfile, mode='w'):
+    def setup_logfile(self, logfile, mode='wb'):
         '''start logging to the given logfile, with timestamps'''
         self.logfile = open(logfile, mode=mode)
 
-    def setup_logfile_raw(self, logfile, mode='w'):
+    def setup_logfile_raw(self, logfile, mode='wb'):
         '''start logging raw bytes to the given logfile, without timestamps'''
         self.logfile_raw = open(logfile, mode=mode)
 
@@ -1201,7 +1211,8 @@ class mavtcp(mavfile):
         mavfile.__init__(self, self.port.fileno(), "tcp:" + device, source_system=source_system, source_component=source_component, use_native=use_native)
 
     def do_connect(self):
-        self.port = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        if sys.platform != 'darwin':
+            self.port = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         retries = self.retries
         if retries <= 0:
             # try to connect at least once:
@@ -1209,6 +1220,8 @@ class mavtcp(mavfile):
         while retries >= 0:
             retries -= 1
             try:
+                if sys.platform == 'darwin':
+                    self.port = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                 self.port.connect(self.destination_addr)
                 break
             except Exception as e:
@@ -1480,6 +1493,7 @@ class mavmmaplog(mavlogfile):
         self.id_to_name = {}
 
         self.instance_offsets = {}
+        self.instance_lengths = {}
 
         self.type_nums = None
 
@@ -1523,7 +1537,13 @@ class mavmmaplog(mavlogfile):
                 m = self.recv_msg()
                 add_message(self.messages, msg.name, m)
                 if m._instance_field is not None:
+                    instance_idx = m.ordered_fieldnames.index(m._instance_field)
                     self.instance_offsets[mtype] = m._instance_offset
+                    alen = m.array_lengths[instance_idx]
+                    if alen > 0:
+                        self.instance_lengths[mtype] = alen
+                    else:
+                        self.instance_lengths[mtype] = 1
 
             if mtype in self.instance_offsets:
                 # populate the messages array with a new instance. This assumes we can get the instance
@@ -1533,11 +1553,29 @@ class mavmmaplog(mavlogfile):
                     # truncated log
                     break
                 self.f.seek(instance_field_ofs)
-                b = self.f.read(1)
-                instance, = struct.unpack('b', b)
+                ilen = self.instance_lengths[mtype]
+                ipad = 0
+                if ilen + (instance_field_ofs - ofs) > mlen-2:
+                    # message is MAVLink2.0 zero truncated
+                    ipad = ilen + (instance_field_ofs - ofs) - (mlen-2)
+                    ilen -= ipad
+                if ilen > 0:
+                    b = self.f.read(ilen)
+                else:
+                    b = bytes([0]*ilen)
+                if ipad > 0:
+                    b += bytes([0]*ipad)
+                if ilen+ipad > 1:
+                    # assume string
+                    while len(b) > 0 and b[-1] == 0:
+                        b = b[:-1]
+                    instance = b.decode('ASCII',errors='ignore').rstrip()
+                else:
+                    instance, = struct.unpack('b', b)
                 mname = self.id_to_name[mtype]
                 if mname in self.messages:
-                    self.messages["%s[%s]" % (mname, str(instance))] = self.messages[mname]
+                    iname = "%s[%s]" % (mname, str(instance))
+                    self.messages[iname] = self.messages[mname]
 
             self.offsets[mtype].append(ofs)
             self.counts[mtype] += 1
@@ -1956,6 +1994,7 @@ mode_mapping_apm = {
     22 : 'QAUTOTUNE',
     23 : 'QACRO',
     24 : 'THERMAL',
+    25 : 'LOITERALTQLAND',
 }
 
 mode_mapping_acm = {
