@@ -3,8 +3,9 @@
 '''
 show GPS lock events in a MAVLink log
 '''
+from __future__ import print_function
 
-import sys, time, os
+import time
 
 from argparse import ArgumentParser
 parser = ArgumentParser(description=__doc__)
@@ -37,7 +38,7 @@ def lock_time(logfile):
             if locked:
                 total_time += time.mktime(t) - start_time
             if total_time > 0:
-                print("Lock time : %u:%02u" % (int(total_time)/60, int(total_time)%60))
+                print("Lock time : %u:%02u" % (int(total_time)//60, int(total_time)%60))
             return total_time
         t = time.localtime(m._timestamp)
         if m.fix_type >= 2 and not locked:
@@ -61,4 +62,4 @@ total = 0.0
 for filename in args.logs:
     total += lock_time(filename)
 
-print("Total time locked: %u:%02u" % (int(total)/60, int(total)%60))
+print("Total time locked: %u:%02u" % (int(total)//60, int(total)%60))
